package src.de.dhbw_ka.probe_pe.darts_counter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Scanner;

/**
 * Class representing the darts game
 */
public class Game {
  /**
   * Instance of the Board
   * 
   * @see Board
   */
  private Board board;
  /**
   * Array of the players playing
   * 
   * @see Player
   */
  private Player[] players;
  /**
   * Index of the player who has the next turn
   */
  private int nextTurnPlayerIndex = 0;

  /**
   * Initiates a new game
   * The game has to be started with `game.start();`
   * 
   * @param board
   * @param players
   */
  public Game(Board board, Player[] players) {
    this.board = board;
    this.players = players;
  }

  /**
   * Checks if the game is ended.
   * 
   * @return true if the game is ended
   */
  private boolean gameEnd() {
    Boolean ended = false;
    for (Player p : players) {
      if (p.getRemainingPoints() == 0)
        ended = true;
    }
    if (players[nextTurnPlayerIndex].getVisitsCount() == 10)
      ended = true;
    return ended;
  }

  /**
   * Gets the player who has won.
   * 
   * @return Winner
   * @see Player
   */
  private Player getWinner() {
    for (Player p : players) {
      if (p.getRemainingPoints() == 0)
        return p;
    }
    return null;
  }

  /**
   * Parses a user input string to a visit
   * 
   * @param input
   * @return visit matching the input string
   * @see Visit
   */
  private Visit inputToVisit(String input) {
    String[] parts = input.split(" ");
    Field[] fields = new Field[parts.length];
    for (int i = 0; i < fields.length; i++)
      fields[i] = board.parseField(parts[i]);
    return new Visit(fields);
  }

  /**
   * Check if a checkout for the remaining points is possible
   * 
   * @param points
   * @return null or a string of fields
   */
  private String checkout(int points) {
    if (points > 170)
      return null;

    Path checkoutsFile = Path.of("src/de/dhbw_ka/probe_pe/darts_counter/checkouts.txt");
    try (BufferedReader reader = Files.newBufferedReader(checkoutsFile, StandardCharsets.UTF_8)) {
      for (int i = 0; i < (points - 1); i++)
        reader.readLine();
      String checkout = reader.readLine();
      if (!checkout.equals("-"))
        return checkout;
    } catch (IOException e) {
      System.err.println("Failed to load checkouts file.");
      e.printStackTrace();
    }
    return null;
  }

  /**
   * Starts the game
   */
  public void start() {
    Scanner in = new Scanner(System.in);
    while (!gameEnd()) {
      Player currentTurnPlayer = this.players[this.nextTurnPlayerIndex];
      nextTurnPlayerIndex = (nextTurnPlayerIndex + 1) % this.players.length;
      System.out.println("Player: " + currentTurnPlayer);
      if (this.checkout(currentTurnPlayer.getRemainingPoints()) != null) {
        System.out.println("Checkout possible with: " + this.checkout(currentTurnPlayer.getRemainingPoints()));
      }
      System.out.print("Enter visit: ");
      String inputString = in.nextLine();
      Visit visit = this.inputToVisit(inputString);
      Boolean success = currentTurnPlayer.addVisit(visit);
      if (success) {
        System.out.println("Scored: " + visit.getValue());
      } else {
        System.out.println("Invalid visit");
      }
      System.out.println("=============================");
    }
    in.close();

    Player winner = this.getWinner();
    if (winner == null) {
      System.out.println("You're too bad for this game");
    } else {
      System.out.println("Game shot and the leg, " + winner.getName());
      this.writeWinnerToFile(winner);
    }
  }

  /**
   * Writes the winner into the highscores.txt file
   * 
   * @param winner
   * @see Player
   */
  private void writeWinnerToFile(Player winner) {
    Path winnerFile = Paths.get("highscores.txt");

    try (BufferedWriter writer = Files.newBufferedWriter(winnerFile, StandardCharsets.UTF_8,
        StandardOpenOption.APPEND)) {
      writer.write(winner.getName() + " won with " + winner.getCountDartsThrown() + " darts.");
      writer.newLine();
    } catch (IOException e) {
      System.err.println("Failed to write highscore file.");
      e.printStackTrace();
    }
  }
}
